"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _path = _interopRequireDefault(require("path"));

var _fs = _interopRequireDefault(require("fs"));

var _os = _interopRequireDefault(require("os"));

var _child_process = require("child_process");

var _chalk = _interopRequireDefault(require("chalk"));

var _readline = _interopRequireDefault(require("readline"));

var _which = _interopRequireDefault(require("../../tools/which"));

var _sexec = _interopRequireDefault(require("../../tools/sexec"));

var _copydirSync = _interopRequireDefault(require("../../tools/copydirSync"));

var _deleteFolderRecursive = _interopRequireDefault(require("../../tools/deleteFolderRecursive"));

var _Configuration = _interopRequireDefault(require("../../Configuration"));

var _constants = _interopRequireDefault(require("../../constants"));

var _Common = _interopRequireDefault(require("../../Common"));

var _Utility = _interopRequireDefault(require("../../Utility"));

/**
 * PM2 Module System.
 * Features:
 * - Installed modules are listed separately from user applications
 * - Always ON, a module is always up along PM2, to stop it, you need to uninstall it
 * - Install a runnable module from NPM/Github/HTTP (require a package.json only)
 * - Some modules add internal PM2 depencencies (like typescript, profiling...)
 * - Internally it uses NPM install (https://docs.npmjs.com/cli/install)
 * - Auto discover script to launch (first it checks the apps field, then bin and finally main attr)
 * - Generate sample module via pm2 module:generate <module_name>
 */
function localStart(PM2, opts, cb) {
  var proc_path = "",
      cmd = ""; // conf = {};

  _Common["default"].printOut(_constants["default"].PREFIX_MSG_MOD + "Installing local module in DEVELOPMENT MODE with WATCH auto restart");

  proc_path = process.cwd();
  cmd = _path["default"].join(proc_path, _constants["default"].DEFAULT_MODULE_JSON);

  _Common["default"].extend(opts, {
    cmd: cmd,
    development_mode: true,
    proc_path: proc_path
  });

  return StartModule(PM2, opts, function (err, dt) {
    if (err) {
      return cb(err);
    }

    _Common["default"].printOut(_constants["default"].PREFIX_MSG_MOD + "Module successfully installed and launched");

    return cb(null, dt);
  });
}

function generateSample(app_name, cb) {
  var rl = _readline["default"].createInterface({
    input: process.stdin,
    output: process.stdout
  });

  function samplize(module_name) {
    var cmd1 = "git clone https://github.com/pm2-hive/sample-module.git " + module_name + "; cd " + module_name + "; rm -rf .git";
    var cmd2 = "cd " + module_name + " ; sed -i \"s:sample-module:" + module_name + ":g\" package.json";
    var cmd3 = "cd " + module_name + " ; npm install";

    _Common["default"].printOut(_constants["default"].PREFIX_MSG_MOD + "Getting sample app");

    (0, _sexec["default"])(cmd1, function (err) {
      if (err) {
        _Common["default"].printError(_constants["default"].PREFIX_MSG_MOD_ERR + err.message);
      }

      (0, _sexec["default"])(cmd2, function (err) {
        console.log("");
        (0, _sexec["default"])(cmd3, function (err) {
          console.log("");

          _Common["default"].printOut(_constants["default"].PREFIX_MSG_MOD + "Module sample created in folder: ", _path["default"].join(process.cwd(), module_name));

          console.log("");

          _Common["default"].printOut("Start module in development mode:");

          _Common["default"].printOut("$ cd " + module_name + "/");

          _Common["default"].printOut("$ pm2 install . ");

          console.log("");

          _Common["default"].printOut("Module Log: ");

          _Common["default"].printOut("$ pm2 logs " + module_name);

          console.log("");

          _Common["default"].printOut("Uninstall module: ");

          _Common["default"].printOut("$ pm2 uninstall " + module_name);

          console.log("");

          _Common["default"].printOut("Force restart: ");

          _Common["default"].printOut("$ pm2 restart " + module_name);

          return cb ? cb() : false;
        });
      });
    });
  }

  if (app_name) {
    return samplize(app_name);
  }

  rl.question(_constants["default"].PREFIX_MSG_MOD + "Module name: ", function (module_name) {
    samplize(module_name);
  });
}

function publish(opts, cb) {
  var rl = _readline["default"].createInterface({
    input: process.stdin,
    output: process.stdout
  });

  var semver = require("semver");

  var package_file = _path["default"].join(process.cwd(), "package.json");

  var package_json = require(package_file);

  package_json.version = semver.inc(package_json.version, "minor");

  _Common["default"].printOut(_constants["default"].PREFIX_MSG_MOD + "Incrementing module to: %s@%s", package_json.name, package_json.version);

  rl.question("Write & Publish? [Y/N]", function (answer) {
    if (answer != "Y") {
      return cb();
    }

    _fs["default"].writeFile(package_file, JSON.stringify(package_json, null, 2), function () {
      // if (err) return cb(err);
      _Common["default"].printOut(_constants["default"].PREFIX_MSG_MOD + "Publishing module - %s@%s", package_json.name, package_json.version);

      (0, _sexec["default"])("npm publish", function (code) {
        _Common["default"].printOut(_constants["default"].PREFIX_MSG_MOD + "Module - %s@%s successfully published", package_json.name, package_json.version);

        _Common["default"].printOut(_constants["default"].PREFIX_MSG_MOD + "Pushing module on Git");

        (0, _sexec["default"])("git add . ; git commit -m \"" + package_json.version + "\"; git push origin master", function (code) {
          _Common["default"].printOut(_constants["default"].PREFIX_MSG_MOD + "Installable with pm2 install %s", package_json.name);

          return cb(null, package_json);
        });
      });
    });
  });
}

function moduleExistInLocalDB(CLI, module_name, cb) {
  var modules = _Configuration["default"].getSync(_constants["default"].MODULE_CONF_PREFIX);

  if (!modules) {
    return cb(false);
  }

  var module_name_only = _Utility["default"].getCanonicModuleName(module_name);

  modules = Object.keys(modules);
  return cb(modules.indexOf(module_name_only) > -1 ? true : false);
}

function install(CLI, module_name, opts, cb) {
  moduleExistInLocalDB(CLI, module_name, function (exists) {
    if (exists) {
      _Common["default"].logMod("Module already installed. Updating.");

      Rollback.backup(module_name);
      return uninstall(CLI, module_name, function () {
        return continueInstall(CLI, module_name, opts, cb);
      });
    }

    return continueInstall(CLI, module_name, opts, cb);
  });
} // Builtin Node Switch


function getNPMCommandLine(module_name, install_path) {
  if ((0, _which["default"])("npm")) {
    return _child_process.spawn.bind(this, _constants["default"].IS_WINDOWS ? "npm.cmd" : "npm", ["install", module_name, "--loglevel=error", "--prefix", "\"".concat(install_path, "\"")], {
      stdio: "inherit",
      env: process.env,
      shell: true
    });
  } else {
    return _child_process.spawn.bind(this, _constants["default"].BUILTIN_NODE_PATH, [_constants["default"].BUILTIN_NPM_PATH, "install", module_name, "--loglevel=error", "--prefix", "\"".concat(install_path, "\"")], {
      stdio: "inherit",
      env: process.env,
      shell: true
    });
  }
}

function continueInstall(CLI, module_name, opts, cb) {
  _Common["default"].printOut(_constants["default"].PREFIX_MSG_MOD + "Calling " + _chalk["default"].bold.red("[NPM]") + " to install " + module_name + " ...");

  var canonic_module_name = _Utility["default"].getCanonicModuleName(module_name);

  var install_path = _path["default"].join(_constants["default"].DEFAULT_MODULE_PATH, canonic_module_name);

  require("mkdirp")(install_path).then(function () {
    process.chdir(_os["default"].homedir());
    var install_instance = getNPMCommandLine(module_name, install_path)();
    install_instance.on("close", finalizeInstall);
    install_instance.on("error", function (err) {
      console.error(err.stack || err);
    });
  });

  function finalizeInstall(code) {
    if (code != 0) {
      // If install has failed, revert to previous module version
      return Rollback.revert(CLI, module_name, function () {
        return cb(new Error("Installation failed via NPM, module has been restored to prev version"));
      });
    }

    _Common["default"].printOut(_constants["default"].PREFIX_MSG_MOD + "Module downloaded");

    var proc_path = _path["default"].join(install_path, "node_modules", canonic_module_name);

    var package_json_path = _path["default"].join(proc_path, "package.json"); // Append default configuration to module configuration


    try {
      var conf = JSON.parse(_fs["default"].readFileSync(package_json_path).toString()).config;

      if (conf) {
        Object.keys(conf).forEach(function (key) {
          _Configuration["default"].setSyncIfNotExist(canonic_module_name + ":" + key, conf[key]);
        });
      }
    } catch (e) {
      _Common["default"].printError(e);
    }

    opts = _Common["default"].extend(opts, {
      cmd: package_json_path,
      development_mode: false,
      proc_path: proc_path
    });

    _Configuration["default"].set(_constants["default"].MODULE_CONF_PREFIX + ":" + canonic_module_name, {
      uid: opts.uid,
      gid: opts.gid
    }, function (err, data) {
      if (err) {
        return cb(err);
      }

      StartModule(CLI, opts, function (err, dt) {
        if (err) {
          return cb(err);
        }

        if (process.env.PM2_PROGRAMMATIC === "true") {
          return cb(null, dt);
        }

        CLI.conf(canonic_module_name, function () {
          _Common["default"].printOut(_constants["default"].PREFIX_MSG_MOD + "Module successfully installed and launched");

          _Common["default"].printOut(_constants["default"].PREFIX_MSG_MOD + "Checkout module options: `$ pm2 conf`");

          return cb(null, dt);
        });
      });
    });
  }
}

function start(PM2, modules, module_name, cb) {
  _Common["default"].printOut(_constants["default"].PREFIX_MSG_MOD + "Starting NPM module " + module_name);

  var install_path = _path["default"].join(_constants["default"].DEFAULT_MODULE_PATH, module_name);

  var proc_path = _path["default"].join(install_path, "node_modules", module_name);

  var package_json_path = _path["default"].join(proc_path, "package.json");

  var opts = {}; // Merge with embedded configuration inside module_conf (uid, gid)

  _Common["default"].extend(opts, modules[module_name]); // Merge meta data to start module properly


  _Common["default"].extend(opts, {
    // package.json path
    cmd: package_json_path,
    // starting mode
    development_mode: false,
    // process cwd
    proc_path: proc_path
  });

  StartModule(PM2, opts, function (err, dt) {
    if (err) {
      console.error(err);
    }

    return cb();
  });
}

function uninstall(CLI, module_name, cb) {
  var module_name_only = _Utility["default"].getCanonicModuleName(module_name);

  var proc_path = _path["default"].join(_constants["default"].DEFAULT_MODULE_PATH, module_name_only);

  _Configuration["default"].unsetSync(_constants["default"].MODULE_CONF_PREFIX + ":" + module_name_only);

  CLI.deleteModule(module_name_only, function (err, data) {
    console.log("Deleting", proc_path);

    if (module_name != "." && proc_path.includes("modules") === true) {
      (0, _deleteFolderRecursive["default"])(proc_path);
    }

    if (err) {
      _Common["default"].printError(err);

      return cb(err);
    }

    return cb(null, data);
  });
}

function getModuleConf(app_name) {
  if (!app_name) {
    throw new Error("No app_name defined");
  }

  var module_conf = _Configuration["default"].getAllSync();

  var additional_env = {};

  if (!module_conf[app_name]) {
    additional_env = {};
    additional_env[app_name] = {};
  } else {
    additional_env = _Common["default"].clone(module_conf[app_name]);
    additional_env[app_name] = JSON.stringify(module_conf[app_name]);
  }

  return additional_env;
}

function StartModule(CLI, opts, cb) {
  if (!opts.cmd && !opts["package"]) {
    throw new Error("module package.json not defined");
  }

  if (!opts.development_mode) {
    opts.development_mode = false;
  }

  var package_json = require(opts.cmd || opts["package"]);
  /**
   * Script file detection
   * 1- *apps* field (default pm2 json configuration)
   * 2- *bin* field
   * 3- *main* field
   */


  if (!package_json.apps && !package_json.pm2) {
    package_json.apps = {};

    if (package_json.bin) {
      var bin = Object.keys(package_json.bin)[0];
      package_json.apps.script = package_json.bin[bin];
    } else if (package_json.main) {
      package_json.apps.script = package_json.main;
    }
  }

  _Common["default"].extend(opts, {
    cwd: opts.proc_path,
    watch: opts.development_mode,
    force_name: package_json.name,
    started_as_module: true
  }); // Start the module


  CLI.start(package_json, opts, function (err, data) {
    if (err) {
      return cb(err);
    }

    if (opts.safe) {
      _Common["default"].printOut(_constants["default"].PREFIX_MSG_MOD + "Monitoring module behavior for potential issue (5secs...)");

      var time = typeof opts.safe == "boolean" ? 3000 : parseInt(opts.safe);
      return setTimeout(function () {
        CLI.describe(package_json.name, function (err, apps) {
          if (err || apps[0].pm2_env.restart_time > 2) {
            return Rollback.revert(CLI, package_json.name, function () {
              return cb(new Error("New Module is instable, restored to previous version"));
            });
          }

          return cb(null, data);
        });
      }, time);
    }

    return cb(null, data);
  });
}

var Rollback = {
  revert: function revert(CLI, module_name, cb) {
    var canonic_module_name = _Utility["default"].getCanonicModuleName(module_name);

    var backup_path = _path["default"].join(require("os").tmpdir(), canonic_module_name);

    var module_path = _path["default"].join(_constants["default"].DEFAULT_MODULE_PATH, canonic_module_name);

    try {
      _fs["default"].statSync(backup_path);
    } catch (e) {
      return cb(new Error("no backup found"));
    }

    _Common["default"].printOut(_constants["default"].PREFIX_MSG_MOD + _chalk["default"].bold.red("[[[[[ Module installation failure! ]]]]]"));

    _Common["default"].printOut(_constants["default"].PREFIX_MSG_MOD + _chalk["default"].bold.red("[RESTORING TO PREVIOUS VERSION]"));

    CLI.deleteModule(canonic_module_name, function () {
      // Delete failing module
      if (module_name.includes("modules") === true) {
        (0, _deleteFolderRecursive["default"])(module_path);
      } // Restore working version


      (0, _copydirSync["default"])(backup_path, _path["default"].join(_constants["default"].DEFAULT_MODULE_PATH, canonic_module_name));

      var proc_path = _path["default"].join(module_path, "node_modules", canonic_module_name);

      var package_json_path = _path["default"].join(proc_path, "package.json"); // Start module


      StartModule(CLI, {
        cmd: package_json_path,
        development_mode: false,
        proc_path: proc_path
      }, cb);
    });
  },
  backup: function backup(module_name) {
    // Backup current module
    var tmpdir = require("os").tmpdir();

    var canonic_module_name = _Utility["default"].getCanonicModuleName(module_name);

    var module_path = _path["default"].join(_constants["default"].DEFAULT_MODULE_PATH, canonic_module_name);

    (0, _copydirSync["default"])(module_path, _path["default"].join(tmpdir, canonic_module_name));
  }
};
var _default = {
  install: install,
  uninstall: uninstall,
  start: start,
  publish: publish,
  generateSample: generateSample,
  localStart: localStart,
  getModuleConf: getModuleConf
};
exports["default"] = _default;