"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _path = _interopRequireDefault(require("path"));

var _fs = _interopRequireDefault(require("fs"));

var _os = _interopRequireDefault(require("os"));

var _child_process = require("child_process");

var _Configuration = _interopRequireDefault(require("../../Configuration"));

var _constants = _interopRequireDefault(require("../../constants"));

var _Common = _interopRequireDefault(require("../../Common"));

var _forEachLimit = _interopRequireDefault(require("async/forEachLimit"));

var _sexec = _interopRequireDefault(require("../../tools/sexec"));

/**
 * Module management to manage tarball packages
 *
 * pm2 install http.tar.gz
 * pm2 uninstall http
 *
 * - the first and only folder in the tarball must be called module (tar zcvf http module/)
 * - a package.json must be present with attribute "name", "version" and "pm2" to declare apps to run
 */
function install(PM2, module_filepath, opts, cb) {
  // Remote file retrieval
  if (module_filepath.includes("http") === true) {
    var target_file = module_filepath.split("/").pop();

    var target_filepath = _path["default"].join(_os["default"].tmpdir(), target_file);

    opts.install_url = module_filepath;
    return retrieveRemote(module_filepath, target_filepath, function (err) {
      if (err) {
        _Common["default"].errMod(err);

        process.exit(1);
      }

      installLocal(PM2, target_filepath, opts, cb);
    });
  } // Local install


  installLocal(PM2, module_filepath, opts, cb);
}

function retrieveRemote(url, dest, cb) {
  _Common["default"].logMod("Retrieving remote package ".concat(url, "..."));

  var wget = (0, _child_process.spawn)("wget", [url, "-O", dest, "-q"], {
    stdio: "inherit",
    env: process.env,
    shell: true
  });
  wget.on("error", function (err) {
    console.error(err.stack || err);
  });
  wget.on("close", function (code) {
    if (code !== 0) {
      return cb(new Error("Could not download"));
    }

    return cb(null);
  });
}

function installLocal(PM2, module_filepath, opts, cb) {
  _Common["default"].logMod("Installing package ".concat(module_filepath)); // Get module name by unpacking the module/package.json only and read the name attribute


  getModuleName(module_filepath, function (err, module_name) {
    if (err) {
      return cb(err);
    }

    _Common["default"].logMod("Module name is ".concat(module_name));

    _Common["default"].logMod("Depackaging module...");

    var install_path = _path["default"].join(_constants["default"].DEFAULT_MODULE_PATH, module_name);

    require("mkdirp").sync(install_path);

    var install_instance = (0, _child_process.spawn)("tar", ["zxf", module_filepath, "-C", install_path, "--strip-components 1"], {
      stdio: "inherit",
      env: process.env,
      shell: true
    });
    install_instance.on("close", function (code) {
      _Common["default"].logMod("Module depackaged in ".concat(install_path));

      if (code == 0) {
        return runInstall(PM2, install_path, module_name, opts, cb);
      }

      return PM2.exitCli(1);
    });
    install_instance.on("error", function (err) {
      console.error(err.stack || err);
    });
  });
}

function deleteModulePath(module_name) {
  // const sanitized = module_name.replace(/\./g, "");
  (0, _child_process.execSync)("rm -r ".concat(_path["default"].join(_constants["default"].DEFAULT_MODULE_PATH, module_name)), {
    silent: true
  });
}

function runInstall(PM2, target_path, module_name, opts, cb) {
  var config_file = _path["default"].join(target_path, "package.json");

  var conf;

  try {
    conf = require(config_file);
    module_name = conf.name;
  } catch (e) {
    _Common["default"].errMod(new Error("Cannot find package.json file with name attribute at least"));
  } // Force with the name in the package.json


  opts.started_as_module = true;
  opts.cwd = target_path;

  if (needPrefix(conf)) {
    opts.name_prefix = module_name;
  }

  if (opts.install) {
    _Common["default"].logMod("Running YARN install...");

    (0, _sexec["default"])("cd ".concat(target_path, " ; yarn install"), {
      silent: false
    }, function (code) {
      // Start apps under "apps" or "pm2" attribute
      _Common["default"].logMod("Starting ".concat(target_path));

      PM2.start(conf, opts, function (err, data) {
        if (err) {
          return cb(err);
        }

        _Configuration["default"].setSync("".concat(_constants["default"].MODULE_CONF_PREFIX_TAR, ":").concat(module_name), {
          source: "tarball",
          install_url: opts.install_url,
          installed_at: Date.now()
        });

        _Common["default"].logMod("Module INSTALLED and STARTED");

        return cb(null, "Module installed & Started");
      });
    });
  } else {
    PM2.start(conf, opts, function (err, data) {
      if (err) {
        return cb(err);
      }

      _Configuration["default"].setSync("".concat(_constants["default"].MODULE_CONF_PREFIX_TAR, ":").concat(module_name), {
        source: "tarball",
        install_url: opts.install_url,
        installed_at: Date.now()
      });

      _Common["default"].logMod("Module INSTALLED and STARTED");

      return cb(null, "Module installed & Started");
    });
  }
}

function start(PM2, module_name, cb) {
  var module_path = _path["default"].join(_constants["default"].DEFAULT_MODULE_PATH, module_name);

  _Common["default"].printOut(_constants["default"].PREFIX_MSG_MOD + "Starting TAR module " + module_name);

  var package_json_path = _path["default"].join(module_path, "package.json");

  var module_conf = _Configuration["default"].getSync("".concat(_constants["default"].MODULE_CONF_PREFIX_TAR, ":").concat(module_name));

  var conf = null;

  try {
    conf = require(package_json_path);
  } catch (e) {
    _Common["default"].printError("Could not find package.json as ".concat(package_json_path));

    return cb();
  }

  var opts = {};
  opts.started_as_module = true;
  opts.cwd = module_path;

  if (module_conf.install_url) {
    opts.install_url = module_conf.install_url;
  }

  if (needPrefix(conf)) {
    opts.name_prefix = module_name;
  }

  PM2.start(conf, opts, function (err, data) {
    if (err) {
      _Common["default"].printError("Could not start ".concat(module_name, " ").concat(module_path));

      return cb();
    }

    _Common["default"].printOut("".concat(_constants["default"].PREFIX_MSG_MOD, " Module ").concat(module_name, " STARTED"));

    return cb();
  });
}
/**
 * Retrieve from module package.json the name of each application
 * delete process and delete folder
 */


function uninstall(PM2, module_name, cb) {
  var module_path = _path["default"].join(_constants["default"].DEFAULT_MODULE_PATH, module_name);

  _Common["default"].logMod("Removing ".concat(module_name, " from auto startup"));

  var pkg;

  try {
    pkg = require(_path["default"].join(module_path, "package.json"));
  } catch (e) {
    _Common["default"].errMod("Could not retrieve module package.json");

    return cb(e);
  }

  var apps = pkg.apps || pkg.pm2;
  apps = [].concat(apps);
  /**
   * Some time a module can have multiple processes
   */

  (0, _forEachLimit["default"])(apps, 1, function (app, next) {
    var app_name;

    if (!app.name) {
      _Common["default"].renderApplicationName(app);
    }

    if (apps.length > 1) {
      app_name = "".concat(module_name, ":").concat(app.name);
    } else if (apps.length == 1 && pkg.name != apps[0].name) {
      app_name = "".concat(module_name, ":").concat(app.name);
    } else {
      app_name = app.name;
    }

    PM2._operate("deleteProcessId", app_name, function () {
      deleteModulePath(module_name);
      next();
    });
  }, function () {
    _Configuration["default"].unsetSync("".concat(_constants["default"].MODULE_CONF_PREFIX_TAR, ":").concat(module_name));

    cb(null);
  });
}
/**
 * Uncompress only module/package.json and retrieve the "name" attribute in the package.json
 */


function getModuleName(module_filepath, cb) {
  var tmp_folder = _path["default"].join(_os["default"].tmpdir(), _constants["default"].MODULE_BASEFOLDER);

  var install_instance = (0, _child_process.spawn)("tar", ["zxf", module_filepath, "-C", _os["default"].tmpdir(), "".concat(_constants["default"].MODULE_BASEFOLDER, "/package.json")], {
    stdio: "inherit",
    env: process.env,
    shell: true
  });
  install_instance.on("close", function (code) {
    try {
      var pkg = JSON.parse(_fs["default"].readFileSync(_path["default"].join(tmp_folder, "package.json"), "utf8"));
      return cb(null, pkg.name);
    } catch (e) {
      return cb(e);
    }
  });
}

function package1(module_path, target_path, cb) {
  var base_folder = _path["default"].dirname(module_path);

  var module_folder_name = _path["default"].basename(module_path);

  var pkg = require(_path["default"].join(module_path, "package.json"));

  var pkg_name = "".concat(module_folder_name, "-v").concat(pkg.version.replace(/\./g, "-"), ".tar.gz");

  var target_fullpath = _path["default"].join(target_path, pkg_name);

  var cmd = "tar zcf ".concat(target_fullpath, " -C ").concat(base_folder, " --transform 's,").concat(module_folder_name, ",module,' ").concat(module_folder_name);

  _Common["default"].logMod("Gziping ".concat(module_path, " to ").concat(target_fullpath));

  var tar = (0, _child_process.exec)(cmd, function (err, sto, ste) {
    if (err) {
      console.log(sto.toString().trim());
      console.log(ste.toString().trim());
    }
  });
  tar.on("close", function (code) {
    cb(code == 0 ? null : code, {
      package_name: pkg_name,
      path: target_fullpath
    });
  });
}

function publish(PM2, folder, cb) {
  var target_folder = folder ? _path["default"].resolve(folder) : process.cwd();
  var pkg;

  try {
    pkg = JSON.parse(_fs["default"].readFileSync(_path["default"].join(target_folder, "package.json")).toString());
  } catch (e) {
    _Common["default"].errMod("".concat(process.cwd(), " module does not contain any package.json"));

    process.exit(1);
  }

  if (!pkg.name) {
    throw new Error("Attribute name should be present");
  }

  if (!pkg.version) {
    throw new Error("Attribute version should be present");
  }

  if (!pkg.pm2 && !pkg.apps) {
    throw new Error("Attribute apps should be present");
  }

  var current_path = target_folder;

  var module_name = _path["default"].basename(current_path);

  var target_path = _os["default"].tmpdir();

  _Common["default"].logMod("Starting publishing procedure for ".concat(module_name, "@").concat(pkg.version));

  package1(current_path, target_path, function (err, res) {
    if (err) {
      _Common["default"].errMod("Can't package, exiting");

      process.exit(1);
    }

    _Common["default"].logMod("Package [".concat(pkg.name, "] created in path ").concat(res.path));

    var data = {
      module_data: {
        file: res.path,
        content_type: "content/gzip"
      },
      id: pkg.name,
      name: pkg.name,
      version: pkg.version
    };
    var uri = "".concat(PM2.pm2_configuration.registry, "/api/v1/modules");

    _Common["default"].logMod("Sending Package to remote ".concat(pkg.name, " ").concat(uri));

    require("needle").post(uri, data, {
      multipart: true
    }, function (err, res, body) {
      if (err) {
        _Common["default"].errMod(err);

        process.exit(1);
      }

      if (res.statusCode !== 200) {
        _Common["default"].errMod("".concat(pkg.name, "-").concat(pkg.version, ": ").concat(res.body.msg));

        process.exit(1);
      }

      _Common["default"].logMod("Module ".concat(module_name, " published under version ").concat(pkg.version));

      process.exit(0);
    });
  });
}

function needPrefix(conf) {
  if (conf.apps && conf.apps.length > 1 || conf.pm2 && conf.pm2.length > 1 || conf.apps.length == 1 && conf.name != conf.apps[0].name) {
    return true;
  }

  return false;
}

var _default = {
  install: install,
  uninstall: uninstall,
  start: start,
  publish: publish,
  package1: package1
};
exports["default"] = _default;