"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _child_process = _interopRequireDefault(require("child_process"));

var _util = require("util");

var _path = _interopRequireDefault(require("path"));

var _runSeries = _interopRequireDefault(require("run-series"));

var _tv = _interopRequireDefault(require("tv4"));

var schema = {
  type: "object",
  properties: {
    user: {
      type: "string",
      minLength: 1
    },
    host: {
      type: ["string", "array"]
    },
    repo: {
      type: "string"
    },
    ref: {
      type: "string"
    },
    path: {
      type: "string"
    },
    appFolderName: {
      type: "string"
    },
    fetch: {
      type: "string"
    }
  },
  required: ["host", "repo", "ref", "path", "appFolderName"]
};
/**
 * Spawn a modified version of visionmedia/deploy
 * @private
 * @param {object} config config to be piped to deploy
 * @param {array}  args custom deploy command-line arguments
 * @param {DeployCallback} cb done callback
 */

function spawn(config, args, cb) {
  var cmd = (0, _util.format)("echo '%j' | \"%s\"", config, require.resolve("./deploy"));
  args = args || [];

  if (args.length > 0) {
    var cmdArgs = args.map(function (arg) {
      return (0, _util.format)("\"%s\"", arg);
    }).join(" ");
    cmd = [cmd, cmdArgs].join(" ");
  }

  var proc = _child_process["default"].spawn("sh", ["-c", cmd], {
    stdio: "inherit"
  });

  var error;
  proc.on("error", function (err) {
    error = err;
  });
  proc.on("close", function (code) {
    if (code === 0) {
      return cb(null, args);
    }

    error = error || new Error((0, _util.format)("Deploy failed with exit code: %s", code));
    error.code = code;
    return cb(error);
  });
}

function clone(obj) {
  return JSON.parse(JSON.stringify(obj));
}

function castArray(arg) {
  return Array.isArray(arg) ? arg : [arg];
}
/**
 * Deploy to a single environment
 * @param {object} deployConfig object containing deploy configs for all environments
 * @param {string} env the name of the environment to deploy to
 * @param {array} args custom deploy command-line arguments
 * @param {DeployCallback} cb done callback
 * @returns {boolean} return value is always `false`
 */


function deployForEnv(deployConfig, env, args, cb) {
  if (!deployConfig[env]) {
    return cb(new Error((0, _util.format)("%s not defined in deploy section", env)));
  }

  var envConfig = clone(deployConfig[env]);

  if (envConfig.ssh_options) {
    envConfig.ssh_options = castArray(envConfig.ssh_options).map(function (option) {
      return (0, _util.format)("-o %s", option);
    }).join(" ");
  }

  var result = _tv["default"].validateResult(envConfig, schema);

  if (!result.valid) {
    return cb(result.error);
  }

  if (process.env.NODE_ENV !== "test") {
    console.log("--> Deploying to %s environment", env);
  }

  if (process.platform !== "win32") {
    envConfig.path = _path["default"].resolve(envConfig.path);
  }

  var hosts = castArray(envConfig.host);
  var jobs = hosts.map(function (host) {
    return function job(done) {
      if (process.env.NODE_ENV !== "test") {
        console.log("--> on host %s", host.host ? host.host : host);
      }

      var config = clone(envConfig);
      config.host = host;
      config["post-deploy"] = prependEnv(config["post-deploy"], config.env);
      spawn(config, args, done);
    };
  });
  (0, _runSeries["default"])(jobs, function (err, result) {
    result = Array.isArray(envConfig.host) ? result : result[0];
    cb(err, result);
  });
  return false;
}

function envToString(env) {
  env = env || {};
  return Object.keys(env).map(function (name) {
    return (0, _util.format)("%s=%s", name.toUpperCase(), env[name]);
  }).join(" ");
}
/**
 * Prepend command with environment variables
 * @private
 * @param {string} cmd command
 * @param {object} env object containing environment variables
 * @returns {string} concatenated shell command
 */


function prependEnv(cmd, env) {
  var envVars = envToString(env);

  if (!envVars) {
    return cmd;
  }

  if (!cmd) {
    return (0, _util.format)("export %s", envVars);
  }

  return (0, _util.format)("export %s && %s", envVars, cmd);
}

var _default = {
  deployForEnv: deployForEnv
};
exports["default"] = _default;