"use strict";

var _interopRequireDefault = require("@babel/runtime/helpers/interopRequireDefault");

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports["default"] = void 0;

var _util = _interopRequireDefault(require("util"));

var _schema = _interopRequireDefault(require("../API/schema"));

/**
 * Copyright 2013 the PM2 project authors. All rights reserved.
 * Use of this source code is governed by a license that
 * can be found in the LICENSE file.
 */

/**
 * Validator of configured file / commander options.
 */
var Config = {
  _errMsgs: {
    "require": "\"%s\" is required",
    "type": "Expect \"%s\" to be a typeof %s, but now is %s",
    "regex": "Verify \"%s\" with regex failed, %s",
    "max": "The maximum of \"%s\" is %s, but now is %s",
    "min": "The minimum of \"%s\" is %s, but now is %s"
  },
  _errors: [],

  /**
   * Schema definition.
   * @returns {exports|*}
   */
  get schema() {
    var _this = this;

    // Cache.
    if (this._schema) {
      return this._schema;
    } // Render aliases.


    this._schema = _schema["default"];

    var _loop = function _loop(k) {
      if (k.indexOf("\\") > 0) {
        return "continue";
      }

      var aliases = [k.split("_").map(function (n, i) {
        if (i != 0 && n && n.length > 1) {
          return n[0].toUpperCase() + n.slice(1);
        }

        return n;
      }).join("")];

      if (_this._schema[k].alias && Array.isArray(_this._schema[k].alias)) {
        // If multiple aliases, merge
        _this._schema[k].alias.forEach(function (alias) {
          aliases.splice(0, 0, alias);
        });
      } else if (_this._schema[k].alias) {
        aliases.splice(0, 0, _this._schema[k].alias);
      }

      _this._schema[k].alias = aliases;
    };

    for (var k in this._schema) {
      var _ret = _loop(k);

      if (_ret === "continue") continue;
    }

    return this._schema;
  },

  /**
   * Filter / Alias options
   */
  filterOptions: function filterOptions(cmd) {
    var conf = {};
    var schema = this.schema;

    var _loop2 = function _loop2(key) {
      var aliases = schema[key].alias;
      aliases && aliases.forEach(function (alias) {
        if (typeof cmd[alias] !== "undefined") {
          conf[key] || (conf[key] = cmd[alias]);
        }
      });
    };

    for (var key in schema) {
      _loop2(key);
    }

    return conf;
  },

  /**
   * Verify JSON configurations.
   * @param {Object} json
   * @returns {{errors: Array, config: {}}}
   */
  validateJSON: function validateJSON(json) {
    var _this2 = this;

    // clone config
    var conf = Object.assign({}, json);
    var res = {};
    this._errors = [];
    var regexKeys = {},
        defines = this.schema;

    var _loop3 = function _loop3(sk) {
      // Pick up RegExp keys.
      if (sk.indexOf("\\") >= 0) {
        regexKeys[sk] = false;
        return "continue";
      }

      var aliases = defines[sk].alias;
      aliases && aliases.forEach(function (alias) {
        conf[sk] || (conf[sk] = json[alias]);
      });
      var val = conf[sk];
      delete conf[sk]; // Validate key-value pairs.

      if (val === undefined || val === null || (val = _this2._valid(sk, val)) === null) {
        // If value is not defined
        // Set default value (via schema.json)
        if (typeof defines[sk]["default"] !== "undefined") {
          res[sk] = defines[sk]["default"];
        }

        return "continue";
      } //console.log(sk, val, val === null, val === undefined);


      res[sk] = val;
    };

    for (var sk in defines) {
      var _ret2 = _loop3(sk);

      if (_ret2 === "continue") continue;
    } // Validate RegExp values.


    var hasRegexKey = false;

    for (var k in regexKeys) {
      hasRegexKey = true;
      regexKeys[k] = new RegExp(k);
    }

    if (hasRegexKey) {
      for (var _k in conf) {
        for (var rk in regexKeys) {
          if (regexKeys[rk].test(_k)) {
            if (this._valid(_k, conf[_k], defines[rk])) {
              res[_k] = conf[_k];
              delete conf[_k];
            }
          }
        }
      }
    }

    return {
      errors: this._errors,
      config: res
    };
  },

  /**
   * Validate key-value pairs by specific schema
   * @param {String} key
   * @param {Mixed} value
   * @param {Object} sch
   * @returns {*}
   * @private
   */
  _valid: function _valid(key, value, _sch) {
    var sch = _sch || this.schema[key];
    var scht = typeof sch.type == "string" ? [sch.type] : sch.type; // Required value.

    var undef = typeof value == "undefined";

    if (this._error(sch.require && undef, "require", key)) {
      return null;
    } // If undefined, make a break.


    if (undef) {
      return null;
    } // Wrap schema types.


    scht = scht.map(function (t) {
      return "[object " + t[0].toUpperCase() + t.slice(1) + "]";
    }); // Typeof value.

    var type = Object.prototype.toString.call(value);
    var nt = "[object Number]"; // Auto parse Number

    if (type != "[object Boolean]" && scht.indexOf(nt) >= 0 && !isNaN(value)) {
      value = parseFloat(value);
      type = nt;
    } // Verify types.


    if (this._error(!~scht.indexOf(type), "type", key, scht.join(" / "), type)) {
      return null;
    } // Verify RegExp if exists.


    if (this._error(type == "[object String]" && sch.regex && !new RegExp(sch.regex).test(value), "regex", key, sch.desc || "should match " + sch.regex)) {
      return null;
    } // Verify maximum / minimum of Number value.


    if (type == "[object Number]") {
      if (this._error(typeof sch.max != "undefined" && value > sch.max, "max", key, sch.max, value)) {
        return null;
      }

      if (this._error(typeof sch.min != "undefined" && value < sch.min, "min", key, sch.min, value)) {
        return null;
      }
    } // If first type is Array, but current is String, try to split them.


    if (scht.length > 1 && type != scht[0] && type == "[object String]") {
      if (scht[0] == "[object Array]") {
        // unfortunately, js does not support lookahead RegExp (/(?<!\\)\s+/) now (until next ver).
        value = value.split(/([\w\-]+\="[^"]*")|([\w\-]+\='[^']*')|"([^"]*)"|'([^']*)'|\s/).filter(function (v) {
          return v && v.trim();
        });
      }
    } // Custom types: sbyte && stime.


    if (sch.ext_type && type == "[object String]" && value.length >= 2) {
      var seed = {
        "sbyte": {
          "G": 1024 * 1024 * 1024,
          "M": 1024 * 1024,
          "K": 1024
        },
        "stime": {
          "h": 60 * 60 * 1000,
          "m": 60 * 1000,
          "s": 1000
        }
      }[sch.ext_type];

      if (seed) {
        value = parseFloat(value.slice(0, -1)) * seed[value.slice(-1)];
      }
    }

    return value;
  },

  /**
   * Wrap errors.
   * @param {Boolean} possible A value indicates whether it is an error or not.
   * @param {String} type
   * @returns {*}
   * @private
   */
  _error: function _error() {
    for (var _len = arguments.length, args = new Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    var possible = args[0],
        type = args[1];

    if (possible) {
      args.splice(0, 2, this._errMsgs[type]);
      this._errors && this._errors.push(_util["default"].format.apply(null, args));
    }

    return possible;
  }
};
var _default = Config;
exports["default"] = _default;